using System.Collections.Generic;
using Mtp2Dav.WebDAVSharp.Server._1d2086a502937936ebc6bfe19cfa15d855be1c31.Adapters;
using Mtp2Dav.WebDAVSharp.Server._1d2086a502937936ebc6bfe19cfa15d855be1c31.Stores;

namespace Mtp2Dav.WebDAVSharp.Server._1d2086a502937936ebc6bfe19cfa15d855be1c31.MethodHandlers
{
    /// <summary>
    ///     This class implements the <c>OPTIONS</c> HTTP method for WebDAV#.
    /// </summary>
    internal class WebDavOptionsMethodHandler : WebDavMethodHandlerBase, IWebDavMethodHandler
    {
        /// <summary>
        ///     Gets the collection of the names of the HTTP methods handled by this instance.
        /// </summary>
        public IEnumerable<string> Names => new[]
        {
            "OPTIONS"
        };

        /// <summary>
        ///     Processes the request.
        /// </summary>
        /// <param name="server">The <see cref="WebDavServer" /> through which the request came in from the client.</param>
        /// <param name="context">
        ///     The
        ///     <see cref="IHttpListenerContext" /> object containing both the request and response
        ///     objects to use.
        /// </param>
        /// <param name="store">The <see cref="IWebDavStore" /> that the <see cref="WebDavServer" /> is hosting.</param>
        public void ProcessRequest(WebDavServer server, IHttpListenerContext context, IWebDavStore store)
        {
            var verbsAllowed = new List<string>
            {
                "OPTIONS",
                "TRACE",
                "GET",
                "HEAD",
                "POST",
                "COPY",
                "PROPFIND",
                "LOCK",
                "UNLOCK"
            };

            var verbsPublic = new List<string>
            {
                "OPTIONS",
                "GET",
                "HEAD",
                "PROPFIND",
                "PROPPATCH",
                "MKCOL",
                "PUT",
                "DELETE",
                "COPY",
                "MOVE",
                "LOCK",
                "UNLOCK"
            };

            foreach (var verb in verbsAllowed)
                context.Response.AppendHeader("Allow", verb);

            foreach (var verb in verbsPublic)
                context.Response.AppendHeader("Public", verb);

            // Sends 200 OK
            context.SendSimpleResponse();
        }
    }
}